package com.caimei.service.auth.impl;

import com.alibaba.fastjson.JSONArray;
import com.caimei.config.FastDfsClient;
import com.caimei.model.en.QrCodeSize;
import com.caimei.model.po.UploadFilePo;
import com.caimei.service.auth.UploadService;
import com.caimei.utils.FileIOUtils;
import com.caimei.utils.ImageUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.URL;
import java.util.*;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

/**
 * Description
 *
 * @author : Aslee
 * @date : 2021/5/11
 */
@Slf4j
@Service
public class UploadServiceImpl implements UploadService {

    @Autowired
    private FastDfsClient client;

    @Value("${spring.profiles.active}")
    private String active;

    @Value("${caimei.imageDomain}")
    private String imageDomain;

    @Override
    public String saveFile(MultipartFile file) throws IOException {
        String originalFilename = file.getOriginalFilename();
        String randomStr = UUID.randomUUID().toString();
        assert originalFilename != null;
        int index = originalFilename.lastIndexOf(".");
        String extName = originalFilename.substring(index);
        String filePath = "/mnt/newdatadrive/data/runtime/jar-instance/zplma/tempFile/";
        if ("dev".equals(active)) {
            filePath = "D:\\WorkSpace\\file\\tempImport\\";
        }
        filePath += randomStr + extName;
        // 临时图片
        File tempFile = new File(filePath);
        file.transferTo(tempFile);
        log.info("【图片上传】>>>>>>>>>>>>>>>>图片临时路径:" + filePath);
        String imageUrl = imageDomain + "/" + client.uploadFile(filePath);
        // 删除临时图片
        boolean delete = tempFile.delete();
        log.info("【图片上传】>>>>>>>>>>>>>>>>删除临时图片:" + delete);
        return imageUrl;
    }


    @Override
    public String saveImageZip(JSONArray imageArr) throws Exception {
        ArrayList<UploadFilePo> fileList = new ArrayList<>();
        for (Object object : imageArr) {
            UploadFilePo file = new UploadFilePo();
            file.setFileUrl(object.toString());
            String imgRandomId = UUID.randomUUID().toString();
            file.setFileName(imgRandomId);
            fileList.add(file);
        }
        HashMap<String, List<UploadFilePo>> map = new HashMap<>(1);
        map.put("图片压缩包", fileList);
        String imageZipPath = createImageZip(map, 5);
        // 上传zip
        String imageUrl = imageDomain + "/" + client.uploadFile(imageZipPath);
        // 删除临时图片
        File zipFile = new File(imageZipPath);
        boolean delete = zipFile.delete();
        log.info("【图片上传】>>>>>>>>>>>>>>>>删除临时图片压缩包:" + delete);
        return imageUrl;
    }

    /**
     * 创建图片压缩包
     *
     * @param fileMap
     * @param type    1机构授权牌，2设备授权牌，3机构二维码，4设备二维码，5资料库图片压缩包
     * @return
     * @throws Exception
     */
    @Override
    public String createImageZip(Map<String, List<UploadFilePo>> fileMap, Integer type) throws Exception {
        String randomStr = UUID.randomUUID().toString();
        String filePath = "/mnt/newdatadrive/data/runtime/jar-instance/zplma/tempImage/";
        if ("dev".equals(active)) {
            filePath = "D:\\uploadZip\\";
        }
        // 压缩文件夹上层临时文件夹路径
        String tempPath = filePath + randomStr;
        // 压缩文件夹路径
        String basePath = "";
        // 生成唯一临时文件夹
        filePath += randomStr + "/";
        // 在临时文件夹中生成要压缩的基础文件夹
        String baseDirName = "";
        switch (type) {
            case 1:
                baseDirName = "机构授权牌";
                break;
            case 2:
                baseDirName = "设备授权牌";
                break;
            case 3:
                baseDirName = "机构二维码";
                break;
            case 4:
                baseDirName = "设备二维码";
                break;
            case 5:
                baseDirName = "图片包";
                break;
            default:
                baseDirName = "";
        }
//        basePath = filePath += baseDirName;
        filePath += baseDirName;
        basePath = filePath;
        File baseDir = new File(basePath);
        if (!baseDir.exists()) {
            baseDir.mkdirs();
        }
        // 往基础文件夹中填充相应数据
        if (1 == type) {
            // 机构授权牌
            List<UploadFilePo> fileList = fileMap.get("机构授权牌");
            for (int i = 0; i < fileList.size(); i++) {
                // 把机构授权牌链接转换为图片
                UploadFilePo file = fileList.get(i);
                String fileUrl = file.getFileUrl();
                URL url = new URL(fileUrl);
                InputStream inputStream = url.openStream();
                BufferedImage authImage = ImageIO.read(inputStream);
                String extName = fileUrl.substring(fileUrl.lastIndexOf(".") + 1);
                String tempFilePath = filePath + "/" + file.getFileName() + "." + extName;
                FileOutputStream outImgStream = new FileOutputStream(tempFilePath);
                ImageIO.write(authImage, extName, outImgStream);
                inputStream.close();
                outImgStream.flush();
                outImgStream.close();
            }
        } else if (2 == type) {
            // 设备授权牌
            for (Map.Entry<String, List<UploadFilePo>> entry : fileMap.entrySet()) {
                String authParty = entry.getKey();
                List<UploadFilePo> fileList = entry.getValue();
                filePath = basePath + "/" + authParty;
                File authDir = new File(filePath);
                if (!authDir.exists()) {
                    authDir.mkdirs();
                }
                for (int i = 0; i < fileList.size(); i++) {
                    // 把设备授权牌链接转换为图片
                    UploadFilePo file = fileList.get(i);
                    String fileUrl = file.getFileUrl();
                    URL url = new URL(fileUrl);
                    InputStream inputStream = url.openStream();
                    BufferedImage authImage = ImageIO.read(inputStream);
                    String extName = fileUrl.substring(fileUrl.lastIndexOf(".") + 1);
                    String tempFilePath = filePath + "/" + file.getFileName() + "." + extName;
                    FileOutputStream outImgStream = new FileOutputStream(tempFilePath);
                    ImageIO.write(authImage, extName, outImgStream);
                    inputStream.close();
                    outImgStream.flush();
                    outImgStream.close();
                }

            }
        } else if (3 == type) {
            // 机构二维码
            List<UploadFilePo> fileList = fileMap.get("机构二维码");
            for (int i = 0; i < fileList.size(); i++) {
                // 把机构二维码链接转换为图片
                UploadFilePo file = fileList.get(i);
                String fileUrl = file.getFileUrl();
                String fileName = file.getFileName();
                BufferedImage qrCodeImage = ImageUtils.createQrCode(fileUrl, QrCodeSize.MLARGE);
                String tempFilePath = filePath + "/" + fileName;
                // 加载背景图片
                BufferedImage backgroundImage = ImageIO.read(new File("/mnt/newdatadrive/data/runtime/jar-instance/zplma/background.png"));
                // 设置文字
                String title = file.getAuthName() + "正品授权";
                String bottom = file.getFileName();
                // 计算二维码和文字的位置
                int qrCodeX = (backgroundImage.getWidth() - qrCodeImage.getWidth()) / 2;
                int qrCodeY = (backgroundImage.getHeight() - qrCodeImage.getHeight()) / 2 + 25;
                // 文字距离图片顶部的距离，可以根据需要调整
                int textY = 30;
                // 创建新图片，使用背景图片的尺寸
                BufferedImage finalImage = new BufferedImage(backgroundImage.getWidth(), backgroundImage.getHeight(), BufferedImage.TYPE_INT_RGB);
                // 获取Graphics2D对象进行绘制
                Graphics2D g2d = finalImage.createGraphics();
                // 抗锯齿
                g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
                g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB);
                g2d.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
                // 绘制背景图片
                g2d.drawImage(backgroundImage, 0, 0, null);
                // 绘制二维码
                g2d.drawImage(qrCodeImage, qrCodeX, qrCodeY, null);
                // 设置字体和颜色 字体大小可根据需要调整
                g2d.setFont(new Font("微软雅黑", Font.BOLD, 20));
                g2d.setColor(Color.WHITE);
                // 绘制文字 如果抬头或者底部字数大于14，需要手动计算换行
                if (title.length() > 14) {
                    // 计算文字的X坐标，使其居中显示
                    FontMetrics metrics = g2d.getFontMetrics();
                    String first = title.substring(0, 14);
                    String second = title.substring(14);
                    int textX = (finalImage.getWidth() - metrics.stringWidth(first)) / 2;
                    int textS = (finalImage.getWidth() - metrics.stringWidth(second)) / 2;
                    g2d.drawString(first, textX, textY);
                    g2d.drawString(second, textS, textY + 20);
                    int textB = (finalImage.getWidth() - metrics.stringWidth(bottom)) / 2;
                    g2d.drawString(bottom, textB, 370);
                } else {
                    // 计算文字的X坐标，使其居中显示
                    FontMetrics metrics = g2d.getFontMetrics();
                    int textX = (finalImage.getWidth() - metrics.stringWidth(title)) / 2;
                    g2d.drawString(title, textX, textY);
                    g2d.drawString(bottom, textX, 370);
                }
                ImageUtils.transferBufferdImage(finalImage, tempFilePath, "png");
                // 清理资源
                g2d.dispose();
            }
        } else if (4 == type) {
            // 设备二维码
            for (Map.Entry<String, List<UploadFilePo>> entry : fileMap.entrySet()) {
                String authParty = entry.getKey();
                List<UploadFilePo> fileList = entry.getValue();
                filePath = basePath + "/" + authParty;
                File authDir = new File(filePath);
                if (!authDir.exists()) {
                    authDir.mkdirs();
                }
                for (int i = 0; i < fileList.size(); i++) {
                    // 把设备授权牌链接转换为图片
                    UploadFilePo file = fileList.get(i);
                    String fileUrl = file.getFileUrl();
                    String fileName = file.getFileName();
                    BufferedImage qrCodeImage = ImageUtils.createQrCode(fileUrl, QrCodeSize.MLARGE);
                    // 0位置仪器名称 1位置sn码
                    String[] split = fileName.split("-");
                    // 加载背景图片
                    BufferedImage backgroundImage = ImageIO.read(new File("/mnt/newdatadrive/data/runtime/jar-instance/zplma/snback.png"));
                    // 设置文字
                    String text = split[0];
                    String bot = "";
                    if (split[1].length() < 3) {
                        bot = "仪器SN码：" + split[1] + "******" + split[1];
                    } else {
                        bot = "仪器SN码：" + split[1].substring(0, 2) + "******" + split[1].substring(0, 3);
                    }
                    // 计算二维码和文字的位置
                    int qrCodeX = (backgroundImage.getWidth() - qrCodeImage.getWidth()) / 2;
                    int qrCodeY = (backgroundImage.getHeight() - qrCodeImage.getHeight()) / 2 + 58;
                    // 文字距离图片顶部的距离，可以根据需要调整
                    int textY = 30;
                    // 创建新图片，使用背景图片的尺寸
                    BufferedImage finalImage = new BufferedImage(backgroundImage.getWidth(), backgroundImage.getHeight(), BufferedImage.TYPE_INT_RGB);
                    // 获取Graphics2D对象进行绘制
                    Graphics2D g2d = finalImage.createGraphics();
                    // 抗锯齿
                    g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
                    g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB);
                    g2d.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
                    // 绘制背景图片
                    g2d.drawImage(backgroundImage, 0, 0, null);
                    // 绘制二维码
                    g2d.drawImage(qrCodeImage, qrCodeX, qrCodeY, null);
                    // 设置字体和颜色 字体大小可根据需要调整
                    g2d.setFont(new Font("微软雅黑", Font.BOLD, 20));
                    g2d.setColor(Color.WHITE);
                    // 计算文字的X坐标，使其居中显示
                    FontMetrics metrics = g2d.getFontMetrics();
                    int textX = (finalImage.getWidth() - metrics.stringWidth(text)) / 2;
                    g2d.drawString(text, textX, textY);
                    // 设置字体和颜色 字体大小可根据需要调整
                    g2d.setFont(new Font("微软雅黑", Font.BOLD, 14));
                    Color customColor = new Color(27, 163, 240);
                    g2d.setColor(customColor);
                    int textB = (finalImage.getWidth() - metrics.stringWidth(bot)) / 2 + 33;
                    g2d.drawString(bot, textB, 90);
                    String tempFilePath = filePath + "/" + fileName;
                    ImageUtils.transferBufferdImage(finalImage, tempFilePath, "png");
                    g2d.dispose();
                }
            }
        }
        // 将文件夹转为压缩文件
        String zipPath = tempPath + "/" + baseDirName + ".zip";
        FileOutputStream zipOut = new FileOutputStream(zipPath);
        FileIOUtils.zipFolder(basePath, zipOut, true);
        return zipPath;
    }

    @Override
    public String saveFileByUrl(String url, String fileName) throws Exception {
        InputStream input = new URL(url).openStream();
        File file = inputStreamToFile(input, fileName);
        String filePath = System.getProperty("java.io.tmpdir") + File.separator + fileName;
        log.info("【图片上传】>>>>>>>>>>>>>>>>图片临时路径:" + filePath);
        String imageUrl = imageDomain + "/" + client.uploadFile(filePath);
        // 删除临时图片
        boolean delete = file.delete();
        log.info("【图片上传】>>>>>>>>>>>>>>>>删除临时图片:" + delete);
        return imageUrl;
    }


    /**
     * 工具类
     * inputStream 转 File
     */
    public static File inputStreamToFile(InputStream ins, String name) throws Exception {
//        File file = new File(System.getProperty("java.io.tmpdir") + File.separator + name);
        File file = new File("D:\\uploadZip\\机构二维码\\" + File.separator + name);
        if (file.exists()) {
            return file;
        }
        OutputStream os = new FileOutputStream(file);
        int bytesRead;
        int len = 8192;
        byte[] buffer = new byte[len];
        while ((bytesRead = ins.read(buffer, 0, len)) != -1) {
            os.write(buffer, 0, bytesRead);
        }
        os.close();
        ins.close();
        return file;
    }
}
